@@ -6,15 +6,41 @@
 
 set -e
 
-if [ -z "$ANDROID_NDK" -o -z "$ANDROID_SDK" -o -z "$ANDROID_ABI" ]; then
+BUILD=
+FETCH=
+case "$1" in
+    --fetch)
+    FETCH=1
+    shift
+    ;;
+    --build)
+    BUILD=1
+    shift
+    ;;
+    *)
+    FETCH=1
+    BUILD=1
+    ;;
+esac
+
+if [ -z "$ANDROID_NDK" -o -z "$ANDROID_SDK" ]; then
    echo "You must define ANDROID_NDK, ANDROID_SDK and ANDROID_ABI before starting."
    echo "They must point to your NDK and SDK directories.\n"
-   echo "ANDROID_ABI should match your ABI: armeabi-v7a, armeabi or ..."
    exit 1
 fi
+
+if [ -z "$ANDROID_ABI" ]; then
+   echo "Please set ANDROID_ABI to your architecture: armeabi-v7a, armeabi, x86 or mips."
+   exit 1
+fi
+
 # try to detect NDK version
 REL=$(grep -o '^r[0-9]*.*' $ANDROID_NDK/RELEASE.TXT 2>/dev/null|cut -b2-)
 case "$REL" in
+    9*)
+        GCCVER=4.8
+        CXXSTL="/"${GCCVER}
+    ;;
     8?*)
         # we don't use 4.4.3 because it doesn't handle threads correctly.
         # TODO : clang?
@@ -79,18 +105,20 @@ export PATH=${NDK_TOOLCHAIN_PATH}:${PATH}
 
 ANDROID_PATH="`pwd`"
 
-# 1/ libvlc, libvlccore and its plugins
-TESTED_HASH=7c52aacbe
-if [ ! -d "vlc" ]; then
-    echo "VLC source not found, cloning"
-    git clone git://git.videolan.org/vlc.git vlc
-    cd vlc
-    git checkout -B android ${TESTED_HASH}
-else
-    echo "VLC source found"
-    cd vlc
-    if ! git cat-file -e ${TESTED_HASH}; then
-        cat << EOF
+if [ ! -z "$FETCH" ]
+then
+    # 1/ libvlc, libvlccore and its plugins
+    TESTED_HASH=5428d0e1f1a
+    if [ ! -d "vlc" ]; then
+        echo "VLC source not found, cloning"
+        git clone git://git.videolan.org/vlc.git vlc
+        cd vlc
+        git checkout -B android ${TESTED_HASH}
+    else
+        echo "VLC source found"
+        cd vlc
+        if ! git cat-file -e ${TESTED_HASH}; then
+            cat << EOF
 ***
 *** Error: Your vlc checkout does not contain the latest tested commit ***
 ***
@@ -105,8 +133,17 @@ git checkout -B android ${TESTED_HASH}
 *** : This will delete any changes you made to the current branch ***
 
 EOF
-        exit 1
+            exit 1
+        fi
     fi
+else
+    cd vlc
+fi
+
+if [ -z "$BUILD" ]
+then
+    echo "Not building anything, please run $0 --build"
+    exit 0
 fi
 
 if [ ${ANDROID_ABI} = "armeabi-v7a" ] ; then
@@ -123,7 +160,7 @@ elif [ ${ANDROID_ABI} = "armeabi" ] ; then
         fi
     fi
 elif [ ${ANDROID_ABI} = "x86" ] ; then
-    EXTRA_CFLAGS="-march=pentium -ffunction-sections -funwind-tables -frtti -fexceptions"
+    EXTRA_CFLAGS="-march=pentium"
 elif [ ${ANDROID_ABI} = "mips" ] ; then
     EXTRA_CFLAGS="-march=mips32 -mtune=mips32r2 -mhard-float"
     # All MIPS Linux kernels since 2.4.4 will trap any unimplemented FPU
@@ -158,8 +195,22 @@ cd ../..
 
 echo "Building the contribs"
 mkdir -p contrib/android
+
+gen_pc_file() {
+    echo "Generating $1 pkg-config file"
+    echo "Name: $1
+Description: $1
+Version: $2
+Libs: -l$1
+Cflags:" > contrib/${TARGET_TUPLE}/lib/pkgconfig/`echo $1|tr 'A-Z' 'a-z'`.pc
+}
+
+mkdir -p contrib/${TARGET_TUPLE}/lib/pkgconfig
+gen_pc_file EGL 1.1
+gen_pc_file GLESv2 2
+
 cd contrib/android
-../bootstrap --host=${TARGET_TUPLE} --disable-disc --disable-sout --enable-small \
+../bootstrap --host=${TARGET_TUPLE} --disable-disc --disable-sout \
     --disable-dca \
     --disable-goom \
     --disable-chromaprint \
@@ -176,7 +227,7 @@ cd contrib/android
     --disable-upnp \
     --disable-gme \
     --disable-tremor \
-    --disable-vorbis \
+    --enable-vorbis \
     --disable-sidplay2 \
     --disable-samplerate \
     --disable-faad2 \
@@ -193,28 +244,34 @@ cd contrib/android
 if [ $# -ne 0 ] && [ "$1" = "release" ]; then
     OPTS=""
     EXTRA_CFLAGS="${EXTRA_CFLAGS} -DNDEBUG "
-    RELEASEFLAG="RELEASE=1"
+    RELEASE=1
 else
     OPTS="--enable-debug"
+    RELEASE=0
 fi
 
 echo "EXTRA_CFLAGS= -g ${EXTRA_CFLAGS}" >> config.mak
 export VLC_EXTRA_CFLAGS="${EXTRA_CFLAGS}"
 
 make fetch
+# We already have zlib available
+[ -e .zlib ] || (mkdir -p zlib; touch .zlib)
+which autopoint >/dev/null || make $MAKEFLAGS .gettext
+export PATH="$PATH:$PWD/../$TARGET_TUPLE/bin"
 make $MAKEFLAGS
 
 cd ../.. && mkdir -p android && cd android
 
 if [ $# -eq 1 ] && [ "$1" = "jni" ]; then
     CLEAN="jniclean"
-    RELEASEFLAG="vlc-android/obj/local/armeabi-v7a/libvlcjni.so"
+    TARGET="vlc-android/obj/local/armeabi-v7a/libvlcjni.so"
 else
     CLEAN="distclean"
     echo "Bootstraping"
     ../bootstrap
     echo "Configuring"
     ${ANDROID_PATH}/configure.sh $OPTS
+    TARGET=
 fi
 
 echo "Building"
@@ -234,7 +291,7 @@ export ANDROID_LIBS=${PWD}/android-libs
 export VLC_BUILD_DIR=vlc/android
 
 make $CLEAN
-make -j1 TARGET_TUPLE=$TARGET_TUPLE PLATFORM_SHORT_ARCH=$PLATFORM_SHORT_ARCH CXXSTL=$CXXSTL $RELEASEFLAG
+make -j1 TARGET_TUPLE=$TARGET_TUPLE PLATFORM_SHORT_ARCH=$PLATFORM_SHORT_ARCH CXXSTL=$CXXSTL RELEASE=$RELEASE $TARGET
 
 # 3/ Environment script
 echo "Generating environment script."
@@ -271,6 +328,7 @@ export ANDROID_SYS_HEADERS_ICS=$ANDROID_SYS_HEADERS_ICS
 export ANDROID_LIBS=$ANDROID_LIBS
 export VLC_BUILD_DIR=$PWD/vlc/android
 export TARGET_TUPLE=$TARGET_TUPLE
+export PATH_HOST=$PATH_HOST
 export PLATFORM_SHORT_ARCH=$PLATFORM_SHORT_ARCH
 EssentialsA
 
