@@ -1,7 +1,7 @@
 /*****************************************************************************
  * MediaDatabase.java
  *****************************************************************************
- * Copyright © 2011-2012 VLC authors and VideoLAN
+ * Copyright © 2011-2014 VLC authors and VideoLAN
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,10 +31,13 @@ import java.util.List;
 import java.util.Locale;
 import java.util.Set;
 
+import org.videolan.libvlc.Media;
+
 import android.content.ContentValues;
 import android.content.Context;
 import android.database.Cursor;
 import android.database.sqlite.SQLiteDatabase;
+import android.database.sqlite.SQLiteException;
 import android.database.sqlite.SQLiteOpenHelper;
 import android.graphics.Bitmap;
 import android.graphics.BitmapFactory;
@@ -111,6 +114,37 @@ public class MediaDatabase {
             super(context, DB_NAME, null, DB_VERSION);
         }
 
+        @Override
+        public SQLiteDatabase getWritableDatabase() {
+            SQLiteDatabase db;
+            try {
+                return super.getWritableDatabase();
+            } catch(SQLiteException e) {
+                try {
+                    db = SQLiteDatabase.openOrCreateDatabase(VLCApplication.getAppContext().getDatabasePath(DB_NAME), null);
+                } catch(SQLiteException e2) {
+                    Log.w(TAG, "SQLite database could not be created! Media library cannot be saved.");
+                    db = SQLiteDatabase.create(null);
+                }
+            }
+            int version = db.getVersion();
+            if (version != DB_VERSION) {
+                db.beginTransaction();
+                try {
+                    if (version == 0) {
+                        onCreate(db);
+                    } else {
+                        onUpgrade(db, version, DB_VERSION);
+                    }
+                    db.setVersion(DB_VERSION);
+                    db.setTransactionSuccessful();
+                } finally {
+                    db.endTransaction();
+                }
+            }
+            return db;
+        }
+
         public void dropMediaTableQuery(SQLiteDatabase db) {
             String query = "DROP TABLE " + MEDIA_TABLE_NAME + ";";
             db.execSQL(query);
@@ -308,7 +342,7 @@ public class MediaDatabase {
         return files;
     }
 
-    public synchronized HashMap<String, Media> getMedias(Context context) {
+    public synchronized HashMap<String, Media> getMedias() {
 
         Cursor cursor;
         HashMap<String, Media> medias = new HashMap<String, Media>();
@@ -339,7 +373,7 @@ public class MediaDatabase {
             if (cursor.moveToFirst()) {
                 do {
                     String location = cursor.getString(12);
-                    Media media = new Media(context, location,
+                    Media media = new Media(location,
                             cursor.getLong(0),      // MEDIA_TIME
                             cursor.getLong(1),      // MEDIA_LENGTH
                             cursor.getInt(2),       // MEDIA_TYPE
@@ -401,12 +435,13 @@ public class MediaDatabase {
         return times;
     }
 
-    public synchronized Media getMedia(Context context, String location) {
+    public synchronized Media getMedia(String location) {
 
         Cursor cursor;
         Media media = null;
 
-        cursor = mDb.query(
+        try {
+            cursor = mDb.query(
                 MEDIA_TABLE_NAME,
                 new String[] {
                         MEDIA_TIME, //0 long
@@ -425,12 +460,16 @@ public class MediaDatabase {
                 MEDIA_LOCATION + "=?",
                 new String[] { location },
                 null, null, null);
+        } catch(IllegalArgumentException e) {
+            // java.lang.IllegalArgumentException: the bind value at index 1 is null
+            return null;
+        }
         if (cursor.moveToFirst()) {
-            media = new Media(context, location,
+            media = new Media(location,
                     cursor.getLong(0),
                     cursor.getLong(1),
                     cursor.getInt(2),
-                    null,
+                    null, // lazy loading, see getPicture()
                     cursor.getString(3),
                     cursor.getString(4),
                     cursor.getString(5),
