@@ -28,7 +28,6 @@ import org.videolan.vlc.AudioServiceController;
 import org.videolan.vlc.R;
 import org.videolan.vlc.Util;
 import org.videolan.vlc.VlcRunnable;
-import org.videolan.vlc.gui.audio.AudioPlayerActivity;
 import org.videolan.vlc.gui.video.VideoPlayerActivity;
 import org.videolan.vlc.interfaces.ISortable;
 
@@ -64,7 +63,7 @@ public class DirectoryViewFragment extends SherlockListFragment implements ISort
     public void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
 
-        mDirectoryAdapter = new DirectoryAdapter();
+        mDirectoryAdapter = new DirectoryAdapter(getActivity());
 
         IntentFilter filter = new IntentFilter();
         filter.addAction(Intent.ACTION_MEDIA_MOUNTED);
@@ -78,6 +77,8 @@ public class DirectoryViewFragment extends SherlockListFragment implements ISort
     @Override
     public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
     {
+        getSherlockActivity().getSupportActionBar().setTitle(R.string.directories);
+
         View v = inflater.inflate(R.layout.directory_view, container, false);
         setListAdapter(mDirectoryAdapter);
         final ListView listView = (ListView)v.findViewById(android.R.id.list);
@@ -142,8 +143,7 @@ public class DirectoryViewFragment extends SherlockListFragment implements ISort
                     });
             alertDialog.show();
         } else if(id == R.id.directory_view_play_audio) {
-            AudioServiceController.getInstance().load(mediaLocation, 0, false, true);
-            AudioPlayerActivity.start(getActivity());
+            AudioServiceController.getInstance().load(mediaLocation, true);
         } else if(id == R.id.directory_view_play_video) {
             VideoPlayerActivity.start(getActivity(), mediaLocation);
         }
@@ -152,10 +152,12 @@ public class DirectoryViewFragment extends SherlockListFragment implements ISort
 
     @Override
     public void onListItemClick(ListView l, View v, int p, long id) {
-        Boolean success = mDirectoryAdapter.browse(p);
-        if(!success) { /* Clicked on a media file */
+        int success = mDirectoryAdapter.browse(p);
+
+        if(success < 0) /* Clicked on a media file */
             openMediaFile(p);
-        }
+        else
+            setSelection(success);
     }
 
     public boolean isRootDirectory () {
@@ -163,7 +165,10 @@ public class DirectoryViewFragment extends SherlockListFragment implements ISort
     }
 
     public void showParentDirectory() {
-        mDirectoryAdapter.browse("..");
+        int success = mDirectoryAdapter.browse("..");
+
+        if(success >= 0)
+            setSelection(success);
     };
 
     private void openMediaFile(int p) {
@@ -175,7 +180,6 @@ public class DirectoryViewFragment extends SherlockListFragment implements ISort
                     || !LibVLC.getExistingInstance().hasVideoTrack(mediaFile)) {
                 List<String> mediaLocations = mDirectoryAdapter.getAllMediaLocations();
                 audioController.load(mediaLocations, mediaLocations.indexOf(mediaFile));
-                AudioPlayerActivity.start(getActivity());
             } else {
                 VideoPlayerActivity.start(getActivity(), mediaFile);
             }
