@@ -1,7 +1,7 @@
 /*****************************************************************************
  * MainActivity.java
  *****************************************************************************
- * Copyright © 2011-2012 VLC authors and VideoLAN
+ * Copyright © 2011-2014 VLC authors and VideoLAN
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,10 @@
 
 package org.videolan.vlc.gui;
 
-import android.os.Message;
+import java.util.Arrays;
+import java.util.HashMap;
+import java.util.List;
+
 import org.videolan.libvlc.LibVlcException;
 import org.videolan.libvlc.LibVlcUtil;
 import org.videolan.vlc.AudioService;
@@ -31,9 +34,14 @@ import org.videolan.vlc.Util;
 import org.videolan.vlc.VLCCallbackTask;
 import org.videolan.vlc.WeakHandler;
 import org.videolan.vlc.gui.SidebarAdapter.SidebarEntry;
+import org.videolan.vlc.gui.audio.AudioAlbumsSongsFragment;
+import org.videolan.vlc.gui.audio.AudioPlayer;
+import org.videolan.vlc.gui.audio.EqualizerFragment;
+import org.videolan.vlc.gui.video.MediaInfoFragment;
+import org.videolan.vlc.gui.video.VideoGridFragment;
 import org.videolan.vlc.gui.video.VideoListAdapter;
 import org.videolan.vlc.interfaces.ISortable;
-import org.videolan.vlc.widget.AudioMiniPlayer;
+import org.videolan.vlc.widget.SlidingPaneLayout;
 
 import android.annotation.TargetApi;
 import android.app.AlertDialog;
@@ -51,10 +59,13 @@ import android.content.res.Configuration;
 import android.os.Build;
 import android.os.Bundle;
 import android.os.Handler;
+import android.os.Message;
 import android.preference.PreferenceManager;
 import android.support.v4.app.Fragment;
+import android.support.v4.app.FragmentActivity;
 import android.support.v4.app.FragmentManager;
 import android.support.v4.app.FragmentTransaction;
+import android.text.InputType;
 import android.util.Log;
 import android.view.Display;
 import android.view.LayoutInflater;
@@ -66,7 +77,6 @@ import android.widget.AdapterView.OnItemClickListener;
 import android.widget.Button;
 import android.widget.CheckBox;
 import android.widget.EditText;
-import android.widget.ImageView;
 import android.widget.ListView;
 import android.widget.ProgressBar;
 import android.widget.TextView;
@@ -85,6 +95,7 @@ public class MainActivity extends SherlockFragmentActivity {
     protected static final String ACTION_SHOW_PROGRESSBAR = "org.videolan.vlc.gui.ShowProgressBar";
     protected static final String ACTION_HIDE_PROGRESSBAR = "org.videolan.vlc.gui.HideProgressBar";
     protected static final String ACTION_SHOW_TEXTINFO = "org.videolan.vlc.gui.ShowTextInfo";
+    public static final String ACTION_SHOW_PLAYER = "org.videolan.vlc.gui.ShowPlayer";
 
     private static final String PREF_SHOW_INFO = "show_info";
     private static final String PREF_FIRST_RUN = "first_run";
@@ -95,13 +106,20 @@ public class MainActivity extends SherlockFragmentActivity {
     private ActionBar mActionBar;
     private SlidingMenu mMenu;
     private SidebarAdapter mSidebarAdapter;
-    private AudioMiniPlayer mAudioPlayer;
+    private AudioPlayer mAudioPlayer;
     private AudioServiceController mAudioController;
+    private SlidingPaneLayout mSlidingPane;
 
     private View mInfoLayout;
     private ProgressBar mInfoProgress;
     private TextView mInfoText;
+    private View mAudioPlayerFilling;
     private String mCurrentFragment;
+    private String mPreviousFragment;
+    private List<String> secondaryFragments = Arrays.asList("albumsSongs", "equalizer",
+                                                            "about", "search", "mediaInfo",
+                                                            "videoGroupList");
+    private HashMap<String, Fragment> mSecondaryFragments = new HashMap<String, Fragment>();
 
     private SharedPreferences mSettings;
 
@@ -165,24 +183,33 @@ public class MainActivity extends SherlockFragmentActivity {
         requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
 
         // Set up the sliding menu
-        setContentView(R.layout.sliding_menu);
-        mMenu = (SlidingMenu) findViewById(R.id.sliding_menu);
+        mMenu = (SlidingMenu) LayoutInflater.from(this).inflate(R.layout.sliding_menu, null);
         changeMenuOffset();
 
+        SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);
+        boolean enableBlackTheme = pref.getBoolean("enable_black_theme", false);
+        if (enableBlackTheme)
+            setTheme(R.style.Theme_VLC_Black);
+
         View v_main = LayoutInflater.from(this).inflate(R.layout.main, null);
-        mMenu.setContent(v_main);
+        setContentView(v_main);
+
+        mSlidingPane = (SlidingPaneLayout) v_main.findViewById(R.id.pane);
+        mSlidingPane.setPanelSlideListener(mPanelSlideListener);
 
         View sidebar = LayoutInflater.from(this).inflate(R.layout.sidebar, null);
         final ListView listView = (ListView)sidebar.findViewById(android.R.id.list);
         listView.setFooterDividersEnabled(true);
-        mSidebarAdapter = new SidebarAdapter();
+        mSidebarAdapter = new SidebarAdapter(this);
         listView.setAdapter(mSidebarAdapter);
         mMenu.setMenu(sidebar);
+        mMenu.attachToActivity(this, SlidingMenu.SLIDING_CONTENT, true);
 
         /* Initialize UI variables */
         mInfoLayout = v_main.findViewById(R.id.info_layout);
         mInfoProgress = (ProgressBar) v_main.findViewById(R.id.info_progress);
         mInfoText = (TextView) v_main.findViewById(R.id.info_text);
+        mAudioPlayerFilling = v_main.findViewById(R.id.audio_player_filling);
 
         /* Set up the action bar */
         prepareActionBar();
@@ -208,6 +235,9 @@ public class MainActivity extends SherlockFragmentActivity {
                  */
                 getSupportFragmentManager().popBackStack(null, FragmentManager.POP_BACK_STACK_INCLUSIVE);
 
+                /* Slide down the audio player */
+                slideDownAudioPlayer();
+
                 /* Switch the fragment */
                 FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
                 ft.replace(R.id.fragment_placeholder, getFragment(entry.id), entry.id);
@@ -228,14 +258,12 @@ public class MainActivity extends SherlockFragmentActivity {
             }
         });
 
-        /* Set up the mini audio player */
-        mAudioPlayer = new AudioMiniPlayer();
+        /* Set up the audio player */
+        mAudioPlayer = new AudioPlayer();
         mAudioController = AudioServiceController.getInstance();
-        mAudioPlayer.setAudioPlayerControl(mAudioController);
-        mAudioPlayer.update();
 
         getSupportFragmentManager().beginTransaction()
-            .replace(R.id.audio_mini_player, mAudioPlayer)
+            .replace(R.id.audio_player, mAudioPlayer)
             .commit();
 
         /* Show info/alpha/beta Warning */
@@ -260,6 +288,7 @@ public class MainActivity extends SherlockFragmentActivity {
         filter.addAction(ACTION_SHOW_PROGRESSBAR);
         filter.addAction(ACTION_HIDE_PROGRESSBAR);
         filter.addAction(ACTION_SHOW_TEXTINFO);
+        filter.addAction(ACTION_SHOW_PLAYER);
         registerReceiver(messageReceiver, filter);
 
         /* Reload the latest preferences */
@@ -278,14 +307,7 @@ public class MainActivity extends SherlockFragmentActivity {
     private void prepareActionBar() {
         mActionBar = getSupportActionBar();
         mActionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_STANDARD);
-        mActionBar.setDisplayOptions(0, ActionBar.DISPLAY_SHOW_TITLE);
         mActionBar.setDisplayHomeAsUpEnabled(true);
-
-        /* Add padding between the home button and the arrow */
-        ImageView home = (ImageView)findViewById(Util.isHoneycombOrLater()
-                ? android.R.id.home : R.id.abs__home);
-        if (home != null)
-            home.setPadding(20, 0, 0, 0);
     }
 
     @Override
@@ -350,6 +372,8 @@ public class MainActivity extends SherlockFragmentActivity {
          */
         if(current == null || (!current.getTag().equals(mCurrentFragment) && found)) {
             Log.d(TAG, "Reloading displayed fragment");
+            if (secondaryFragments.contains(mCurrentFragment))
+                mCurrentFragment = "video";
             Fragment ff = getFragment(mCurrentFragment);
             FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
             ft.replace(R.id.fragment_placeholder, ff, mCurrentFragment);
@@ -400,6 +424,11 @@ public class MainActivity extends SherlockFragmentActivity {
             mMenu.showContent();
             return;
         }
+
+        // Slide down the audio player if it is shown entirely.
+        if (slideDownAudioPlayer())
+            return;
+
         // If it's the directory view, a "backpressed" action shows a parent.
         if (mCurrentFragment.equals("directories")) {
             DirectoryViewFragment directoryView = (DirectoryViewFragment) getFragment(mCurrentFragment);
@@ -408,6 +437,13 @@ public class MainActivity extends SherlockFragmentActivity {
                 return;
             }
         }
+
+        // If it's the albums songs fragment, we leave it.
+        if (secondaryFragments.contains(mCurrentFragment)) {
+            popSecondaryFragment();
+            return;
+        }
+
         super.onBackPressed();
     }
 
@@ -416,6 +452,92 @@ public class MainActivity extends SherlockFragmentActivity {
         return mSidebarAdapter.fetchFragment(id);
     }
 
+    private static void ShowFragment(FragmentActivity activity, String tag, Fragment fragment) {
+        if (fragment == null) {
+            Log.e(TAG, "Cannot show a null fragment, ShowFragment("+tag+") aborted.");
+            return;
+        }
+
+        FragmentManager fm = activity.getSupportFragmentManager();
+
+        //abort if fragment is already the current one
+        Fragment current = fm.findFragmentById(R.id.fragment_placeholder);
+        if(current != null && current.getTag().equals(tag))
+            return;
+
+        //try to pop back if the fragment is already on the backstack
+        if (fm.popBackStackImmediate(tag, 0))
+            return;
+
+        //fragment is not there yet, spawn a new one
+        FragmentTransaction ft = fm.beginTransaction();
+        ft.setCustomAnimations(R.anim.anim_enter_right, R.anim.anim_leave_left, R.anim.anim_enter_left, R.anim.anim_leave_right);
+        ft.replace(R.id.fragment_placeholder, fragment, tag);
+        ft.addToBackStack(tag);
+        ft.commit();
+    }
+
+    /**
+     * Fetch a secondary fragment.
+     * @param id the fragment id
+     * @return the fragment.
+     */
+    public Fragment fetchSecondaryFragment(String id) {
+        if (mSecondaryFragments.containsKey(id)
+            && mSecondaryFragments.get(id) != null)
+            return mSecondaryFragments.get(id);
+
+        Fragment f;
+        if (id.equals("albumsSongs")) {
+            f = new AudioAlbumsSongsFragment();
+        } else if(id.equals("equalizer")) {
+            f = new EqualizerFragment();
+        } else if(id.equals("about")) {
+            f = new AboutFragment();
+        } else if(id.equals("search")) {
+            f = new SearchFragment();
+        } else if(id.equals("mediaInfo")) {
+            f = new MediaInfoFragment();
+        } else if(id.equals("videoGroupList")) {
+            f = new VideoGridFragment();
+        }
+        else {
+            throw new IllegalArgumentException("Wrong fragment id.");
+        }
+        f.setRetainInstance(true);
+        mSecondaryFragments.put(id, f);
+        return f;
+    }
+
+    /**
+     * Show a secondary fragment.
+     */
+    public Fragment showSecondaryFragment(String fragmentTag) {
+        // Slide down the audio player if needed.
+        slideDownAudioPlayer();
+
+        // Do not show the new fragment if the requested fragment is already shown.
+        if (mCurrentFragment.equals(fragmentTag))
+            return null;
+
+        if (!secondaryFragments.contains(mCurrentFragment))
+            mPreviousFragment = mCurrentFragment;
+
+        mCurrentFragment = fragmentTag;
+        Fragment frag = fetchSecondaryFragment(mCurrentFragment);
+        ShowFragment(this, mCurrentFragment, frag);
+        return frag;
+    }
+
+    /**
+     * Hide the current secondary fragment.
+     */
+    public void popSecondaryFragment() {
+        FragmentManager fragmentManager = getSupportFragmentManager();
+        fragmentManager.popBackStackImmediate(null, FragmentManager.POP_BACK_STACK_INCLUSIVE);
+        mCurrentFragment = mPreviousFragment;
+    }
+
     /** Create menu from XML
      */
     @Override
@@ -429,6 +551,24 @@ public class MainActivity extends SherlockFragmentActivity {
         return super.onCreateOptionsMenu(menu);
     }
 
+    @Override
+    public boolean onPrepareOptionsMenu (Menu menu) {
+        Fragment current = getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder);
+        // Disable the sort option if we can't use it on the current fragment.
+        if (current == null || !(current instanceof ISortable)) {
+            menu.findItem(R.id.ml_menu_sortby).setEnabled(false);
+            menu.findItem(R.id.ml_menu_sortby).setVisible(false);
+        }
+        else {
+            menu.findItem(R.id.ml_menu_sortby).setEnabled(true);
+            menu.findItem(R.id.ml_menu_sortby).setVisible(true);
+        }
+        // Enable the clear search history function for the search fragment.
+        if (mCurrentFragment.equals("search"))
+            menu.findItem(R.id.search_clear_history).setVisible(true);
+        return true;
+    }
+
     @Override
     public void onConfigurationChanged(Configuration newConfig) {
         super.onConfigurationChanged(newConfig);
@@ -437,9 +577,10 @@ public class MainActivity extends SherlockFragmentActivity {
 
     @Override
     public boolean onSearchRequested() {
-        Intent intent = new Intent(this, SearchActivity.class);
-        startActivity(intent);
-        return false;
+        if (mCurrentFragment.equals("search"))
+            ((SearchFragment)fetchSecondaryFragment("search")).onSearchKeyPressed();
+        showSecondaryFragment("search");
+        return true;
     }
 
     /**
@@ -465,14 +606,16 @@ public class MainActivity extends SherlockFragmentActivity {
                 break;
             // About
             case R.id.ml_menu_about:
-                intent = new Intent(this, AboutActivity.class);
-                startActivity(intent);
+                showSecondaryFragment("about");
                 break;
             // Preferences
             case R.id.ml_menu_preferences:
                 intent = new Intent(this, PreferencesActivity.class);
                 startActivityForResult(intent, ACTIVITY_RESULT_PREFERENCES);
                 break;
+            case R.id.ml_menu_equalizer:
+                showSecondaryFragment("equalizer");
+                break;
             // Refresh
             case R.id.ml_menu_refresh:
                 // TODO: factor this into each fragment
@@ -495,15 +638,27 @@ public class MainActivity extends SherlockFragmentActivity {
                 onOpenMRL();
                 break;
             case R.id.ml_menu_search:
-            	onSearchRequested();
-            	break;
+                onSearchRequested();
+                break;
             case android.R.id.home:
+                // Slide down the audio player.
+                if (slideDownAudioPlayer())
+                    break;
+
+                // If it's the albums songs view, a "backpressed" action shows .
+                if (secondaryFragments.contains(mCurrentFragment)) {
+                    popSecondaryFragment();
+                    break;
+                }
                 /* Toggle the sidebar */
                 if(mMenu.isMenuShowing())
                     mMenu.showContent();
                 else
                     mMenu.showMenu();
                 break;
+            case R.id.search_clear_history:
+                ((SearchFragment)fetchSecondaryFragment("search")).clearSearchHistory();
+                break;
         }
         return super.onOptionsItemSelected(item);
     }
@@ -514,6 +669,11 @@ public class MainActivity extends SherlockFragmentActivity {
         if (requestCode == ACTIVITY_RESULT_PREFERENCES) {
             if (resultCode == PreferencesActivity.RESULT_RESCAN)
                 MediaLibrary.getInstance(this).loadMediaItems(this, true);
+            else if (resultCode == PreferencesActivity.RESULT_RESTART) {
+                Intent intent = getIntent();
+                finish();
+                startActivity(intent);
+            }
         }
     }
 
@@ -586,6 +746,8 @@ public class MainActivity extends SherlockFragmentActivity {
                         mHandler.sendMessageDelayed(m, 300);
                     }
                 }
+            } else if (action.equalsIgnoreCase(ACTION_SHOW_PLAYER)) {
+                showAudioPlayer();
             }
         }
     };
@@ -642,6 +804,7 @@ public class MainActivity extends SherlockFragmentActivity {
     private void onOpenMRL() {
         AlertDialog.Builder b = new AlertDialog.Builder(this);
         final EditText input = new EditText(this);
+        input.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_URI);
         b.setTitle(R.string.open_mrl_dialog_title);
         b.setMessage(R.string.open_mrl_dialog_msg);
         b.setView(input);
@@ -663,7 +826,7 @@ public class MainActivity extends SherlockFragmentActivity {
                        * (for example, RTSP and TS streaming) where ES can be
                        * dynamically adapted rather than a simple scan.
                        */
-                      c.append(s);
+                      c.load(s, false);
                     }
                 };
                 task.execute();
@@ -677,4 +840,78 @@ public class MainActivity extends SherlockFragmentActivity {
                 }});
         b.show();
     }
+
+    /**
+     * Show the audio player.
+     */
+    public void showAudioPlayer() {
+        // Open the pane only if is entirely opened.
+        if (mSlidingPane.getState() == mSlidingPane.STATE_OPENED_ENTIRELY)
+            mSlidingPane.openPane();
+        mAudioPlayerFilling.setVisibility(View.VISIBLE);
+    }
+
+    /**
+     * Slide down the audio player.
+     * @return true on success else false.
+     */
+    public boolean slideDownAudioPlayer() {
+        if (mSlidingPane.getState() == mSlidingPane.STATE_CLOSED) {
+            mSlidingPane.openPane();
+            return true;
+        }
+        return false;
+    }
+
+    /**
+     * Slide up and down the audio player depending on its current state.
+     */
+    public void slideUpOrDownAudioPlayer() {
+        if (mSlidingPane.getState() == mSlidingPane.STATE_CLOSED)
+            mSlidingPane.openPane();
+        else if (mSlidingPane.getState() == mSlidingPane.STATE_OPENED)
+            mSlidingPane.closePane();
+    }
+
+    /**
+     * Hide the audio player.
+     */
+    public void hideAudioPlayer() {
+        mSlidingPane.openPaneEntirely();
+        mAudioPlayerFilling.setVisibility(View.GONE);
+    }
+
+    private final SlidingPaneLayout.PanelSlideListener mPanelSlideListener
+        = new SlidingPaneLayout.PanelSlideListener() {
+
+            @Override
+            public void onPanelSlide(float slideOffset) {
+                if (slideOffset <= 0.1)
+                    getSupportActionBar().hide();
+                else
+                    getSupportActionBar().show();
+            }
+
+            @Override
+            public void onPanelOpened() {
+                int resId = Util.getResourceFromAttribute(MainActivity.this, R.attr.mini_player_top_shadow);
+                if (resId != 0)
+                    mSlidingPane.setShadowResource(resId);
+                mAudioPlayer.setHeaderVisibilities(false, false, true, true, true);
+                mMenu.setSlidingEnabled(true);
+            }
+
+            @Override
+            public void onPanelOpenedEntirely() {
+                mSlidingPane.setShadowDrawable(null);
+                mMenu.setSlidingEnabled(true);
+            }
+
+            @Override
+            public void onPanelClosed() {
+                mAudioPlayer.setHeaderVisibilities(true, true, false, false, false);
+                mMenu.setSlidingEnabled(false);
+            }
+
+    };
 }
