@@ -26,9 +26,11 @@ import java.util.concurrent.BrokenBarrierException;
 import java.util.concurrent.CyclicBarrier;
 
 import org.videolan.android.ui.SherlockGridFragment;
+import org.videolan.libvlc.LibVlcUtil;
+import org.videolan.libvlc.Media;
 import org.videolan.vlc.AudioServiceController;
-import org.videolan.vlc.Media;
 import org.videolan.vlc.MediaDatabase;
+import org.videolan.vlc.MediaGroup;
 import org.videolan.vlc.MediaLibrary;
 import org.videolan.vlc.R;
 import org.videolan.vlc.Thumbnailer;
@@ -36,8 +38,7 @@ import org.videolan.vlc.Util;
 import org.videolan.vlc.VlcRunnable;
 import org.videolan.vlc.WeakHandler;
 import org.videolan.vlc.gui.CommonDialogs;
-import org.videolan.vlc.gui.PreferencesActivity;
-import org.videolan.vlc.gui.audio.AudioPlayerActivity;
+import org.videolan.vlc.gui.MainActivity;
 import org.videolan.vlc.interfaces.ISortable;
 
 import android.annotation.TargetApi;
@@ -47,7 +48,6 @@ import android.content.BroadcastReceiver;
 import android.content.Context;
 import android.content.Intent;
 import android.content.IntentFilter;
-import android.content.SharedPreferences;
 import android.content.res.Configuration;
 import android.os.Build;
 import android.os.Bundle;
@@ -59,6 +59,7 @@ import android.util.Log;
 import android.view.ContextMenu;
 import android.view.ContextMenu.ContextMenuInfo;
 import android.view.LayoutInflater;
+import android.view.Menu;
 import android.view.MenuInflater;
 import android.view.MenuItem;
 import android.view.View;
@@ -89,8 +90,10 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
     private static final int LIST_STRETCH_MODE = GridView.STRETCH_COLUMN_WIDTH;
 
     protected LinearLayout mLayoutFlipperLoading;
+    protected GridView mGridView;
     protected TextView mTextViewNomedia;
     protected Media mItemToUpdate;
+    protected String mGroup;
     protected final CyclicBarrier mBarrier = new CyclicBarrier(2);
 
     private VideoListAdapter mVideoAdapter;
@@ -98,12 +101,20 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
     private Thumbnailer mThumbnailer;
     private VideoGridAnimator mAnimator;
 
+    private AudioServiceController mAudioController;
+
+    // Gridview position saved in onPause()
+    private int mGVFirstVisiblePos;
+
     /* All subclasses of Fragment must include a public empty constructor. */
     public VideoGridFragment() { }
 
     @Override
     public void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
+
+        mAudioController = AudioServiceController.getInstance();
+
         mVideoAdapter = new VideoListAdapter(getActivity(), this);
         mMediaLibrary = MediaLibrary.getInstance(getActivity());
         setListAdapter(mVideoAdapter);
@@ -117,11 +128,14 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
     @Override
     public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState)
     {
+        getSherlockActivity().getSupportActionBar().setTitle(R.string.video);
+
         View v = inflater.inflate(R.layout.video_grid, container, false);
 
         // init the information for the scan (1/2)
         mLayoutFlipperLoading = (LinearLayout) v.findViewById(R.id.layout_flipper_loading);
         mTextViewNomedia = (TextView) v.findViewById(R.id.textview_nomedia);
+        mGridView = (GridView) v.findViewById(android.R.id.list);
 
         return v;
     }
@@ -141,13 +155,13 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
             actionScanStart(getActivity().getApplicationContext());
         }
 
-        updateList();
         mAnimator = new VideoGridAnimator(getGridView());
     }
 
     @Override
     public void onPause() {
         super.onPause();
+        mGVFirstVisiblePos = mGridView.getFirstVisiblePosition();
         mMediaLibrary.removeUpdateHandler(mHandler);
 
         /* Stop the thumbnailer */
@@ -158,13 +172,13 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
     @Override
     public void onResume() {
         super.onResume();
-        //Get & highlight the last media
-        SharedPreferences preferences = getActivity().getSharedPreferences(PreferencesActivity.NAME, Context.MODE_PRIVATE);
-        String lastPath = preferences.getString(PreferencesActivity.LAST_MEDIA, null);
+        //Get & set times
         HashMap<String, Long> times = MediaDatabase.getInstance(getActivity()).getVideoTimes(getActivity());
-        mVideoAdapter.setLastMedia(lastPath, times);
+        mVideoAdapter.setTimes(times);
         mVideoAdapter.notifyDataSetChanged();
+        updateList();
         mMediaLibrary.addUpdateHandler(mHandler);
+        mGridView.setSelection(mGVFirstVisiblePos);
         updateViewMode();
         mAnimator.animate();
 
@@ -193,14 +207,13 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
         if (activity == null)
             return true;
 
-        final GridView grid = (GridView)v.findViewById(android.R.id.list);
-
         DisplayMetrics outMetrics = new DisplayMetrics();
         activity.getWindowManager().getDefaultDisplay().getMetrics(outMetrics);
 
         final int itemWidth = Util.convertDpToPx(GRID_ITEM_WIDTH_DP);
         final int horizontalspacing = Util.convertDpToPx(GRID_HORIZONTAL_SPACING_DP);
-        final int width = grid.getPaddingLeft() + grid.getPaddingRight() + horizontalspacing + (itemWidth * 2);
+        final int width = mGridView.getPaddingLeft() + mGridView.getPaddingRight()
+                + horizontalspacing + (itemWidth * 2);
         if (width < outMetrics.widthPixels)
             return true;
         return false;
@@ -212,30 +225,29 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
             return;
         }
 
-        GridView gv = (GridView)getView().findViewById(android.R.id.list);
-
         // Compute the left/right padding dynamically
         DisplayMetrics outMetrics = new DisplayMetrics();
         getActivity().getWindowManager().getDefaultDisplay().getMetrics(outMetrics);
         int sidePadding = (int) (outMetrics.widthPixels / 100 * Math.pow(outMetrics.density, 3) / 2);
         sidePadding = Math.max(0, Math.min(100, sidePadding));
-        gv.setPadding(sidePadding, gv.getPaddingTop(), sidePadding, gv.getPaddingBottom());
+        mGridView.setPadding(sidePadding, mGridView.getPaddingTop(),
+                sidePadding, mGridView.getPaddingBottom());
 
         // Select between grid or list
         if (hasSpaceForGrid(getView())) {
             Log.d(TAG, "Switching to grid mode");
-            gv.setNumColumns(GridView.AUTO_FIT);
-            gv.setStretchMode(GRID_STRETCH_MODE);
-            gv.setHorizontalSpacing(Util.convertDpToPx(GRID_HORIZONTAL_SPACING_DP));
-            gv.setVerticalSpacing(Util.convertDpToPx(GRID_VERTICAL_SPACING_DP));
-            gv.setColumnWidth(Util.convertDpToPx(GRID_ITEM_WIDTH_DP));
+            mGridView.setNumColumns(GridView.AUTO_FIT);
+            mGridView.setStretchMode(GRID_STRETCH_MODE);
+            mGridView.setHorizontalSpacing(Util.convertDpToPx(GRID_HORIZONTAL_SPACING_DP));
+            mGridView.setVerticalSpacing(Util.convertDpToPx(GRID_VERTICAL_SPACING_DP));
+            mGridView.setColumnWidth(Util.convertDpToPx(GRID_ITEM_WIDTH_DP));
             mVideoAdapter.setListMode(false);
         } else {
             Log.d(TAG, "Switching to list mode");
-            gv.setNumColumns(1);
-            gv.setStretchMode(LIST_STRETCH_MODE);
-            gv.setHorizontalSpacing(LIST_HORIZONTAL_SPACING_DP);
-            gv.setVerticalSpacing(Util.convertDpToPx(LIST_VERTICAL_SPACING_DP));
+            mGridView.setNumColumns(1);
+            mGridView.setStretchMode(LIST_STRETCH_MODE);
+            mGridView.setHorizontalSpacing(LIST_HORIZONTAL_SPACING_DP);
+            mGridView.setVerticalSpacing(Util.convertDpToPx(LIST_VERTICAL_SPACING_DP));
             mVideoAdapter.setListMode(true);
         }
     }
@@ -252,48 +264,51 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
 
     @Override
     public void onGridItemClick(GridView l, View v, int position, long id) {
-        playVideo(position, false);
+        Media media = (Media) getListAdapter().getItem(position);
+        if (media instanceof MediaGroup) {
+            MainActivity activity = (MainActivity)getActivity();
+            VideoGridFragment frag = (VideoGridFragment)activity.showSecondaryFragment("videoGroupList");
+            frag.setGroup(media.getTitle());
+        }
+        else
+            playVideo(media, false);
         super.onGridItemClick(l, v, position, id);
     }
 
-    protected void playVideo(int position, boolean fromStart) {
-        Media item = (Media) getListAdapter().getItem(position);
-        VideoPlayerActivity.start(getActivity(), item.getLocation(), fromStart);
+    protected void playVideo(Media media, boolean fromStart) {
+        VideoPlayerActivity.start(getActivity(), media.getLocation(), fromStart);
     }
 
-    protected void playAudio(int position) {
-        Media item = (Media) getListAdapter().getItem(position);
-        AudioServiceController.getInstance().load(item.getLocation(), 0, false, true);
-        AudioPlayerActivity.start(getActivity());
+    protected void playAudio(Media media) {
+        mAudioController.load(media.getLocation(), true);
     }
 
     private boolean handleContextItemSelected(MenuItem menu, int position) {
+        Media media = mVideoAdapter.getItem(position);
         switch (menu.getItemId())
         {
-        case R.id.video_list_play:
-            playVideo(position, false);
-            return true;
         case R.id.video_list_play_from_start:
-            playVideo(position, true);
+            playVideo(media, true);
             return true;
         case R.id.video_list_play_audio:
-            playAudio(position);
+            playAudio(media);
             return true;
         case R.id.video_list_info:
-            Intent intent = new Intent(getActivity(), MediaInfoActivity.class);
-            intent.putExtra("itemLocation",
-                    mVideoAdapter.getItem(position).getLocation());
-            startActivity(intent);
+            MainActivity activity = (MainActivity)getActivity();
+            MediaInfoFragment frag = (MediaInfoFragment)activity.showSecondaryFragment("mediaInfo");
+            frag.setMediaLocation(media.getLocation());
             return true;
         case R.id.video_list_delete:
-            final int positionDelete = position;
             AlertDialog alertDialog = CommonDialogs.deleteMedia(
                     getActivity(),
-                    mVideoAdapter.getItem(positionDelete).getLocation(),
-                    new VlcRunnable() {
+                    media.getLocation(),
+                    new VlcRunnable(media) {
                         @Override
                         public void run(Object o) {
-                            mVideoAdapter.remove(mVideoAdapter.getItem(positionDelete));
+                            Media media = (Media) o;
+                            mMediaLibrary.getMediaItems().remove(media);
+                            mVideoAdapter.remove(media);
+                            mAudioController.removeLocation(media.getLocation());
                         }
                     });
             alertDialog.show();
@@ -304,8 +319,22 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
 
     @Override
     public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
+        // Do not show the menu of media group.
+        AdapterContextMenuInfo info = (AdapterContextMenuInfo)menuInfo;
+        Media media = mVideoAdapter.getItem(info.position);
+        if (media instanceof MediaGroup)
+            return;
         MenuInflater inflater = getActivity().getMenuInflater();
         inflater.inflate(R.menu.video_list, menu);
+        setContextMenuItems(menu, media);
+    }
+
+    private void setContextMenuItems(Menu menu, Media media) {
+        long lastTime = media.getTime();
+        if (lastTime > 0) {
+            MenuItem playFromStart = menu.findItem(R.id.video_list_play_from_start);
+            playFromStart.setVisible(true);
+        }
     }
 
     @Override
@@ -318,7 +347,7 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
 
     @TargetApi(Build.VERSION_CODES.HONEYCOMB)
     public void onContextPopupMenu(View anchor, final int position) {
-        if (!Util.isHoneycombOrLater()) {
+        if (!LibVlcUtil.isHoneycombOrLater()) {
             // Call the "classic" context menu
             anchor.performLongClick();
             return;
@@ -326,6 +355,8 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
 
         PopupMenu popupMenu = new PopupMenu(getActivity(), anchor);
         popupMenu.getMenuInflater().inflate(R.menu.video_list, popupMenu.getMenu());
+        Media media = mVideoAdapter.getItem(position);
+        setContextMenuItems(popupMenu.getMenu(), media);
         popupMenu.setOnMenuItemClickListener(new OnMenuItemClickListener() {
             @Override
             public boolean onMenuItemClick(MenuItem item) {
@@ -385,10 +416,20 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
         mVideoAdapter.clear();
 
         if (itemList.size() > 0) {
-            for (Media item : itemList) {
-                if (item.getType() == Media.TYPE_VIDEO) {
-                    mVideoAdapter.add(item);
-                    if (mThumbnailer != null && item.getPicture() == null && !item.isPictureParsed())
+            if (mGroup != null || itemList.size() <= 10) {
+                for (Media item : itemList) {
+                    if (mGroup == null || item.getTitle().startsWith(mGroup)) {
+                        mVideoAdapter.add(item);
+                        if (mThumbnailer != null)
+                            mThumbnailer.addJob(item);
+                    }
+                }
+            }
+            else {
+                List<MediaGroup> groups = MediaGroup.group(itemList);
+                for (MediaGroup item : groups) {
+                    mVideoAdapter.add(item.getMedia());
+                    if (mThumbnailer != null)
                         mThumbnailer.addJob(item);
                 }
             }
@@ -406,6 +447,10 @@ public class VideoGridFragment extends SherlockGridFragment implements ISortable
         mHandler.sendEmptyMessage(UPDATE_ITEM);
     }
 
+    public void setGroup(String prefix) {
+        mGroup = prefix;
+    }
+
     public void await() throws InterruptedException, BrokenBarrierException {
         mBarrier.await();
     }
